/*
	*
	* This file is taken from CoreStats <https://gitlab.com/cubocore/corestats>
	* Further modifications may be done as per the requirements of DesQ
	* All bug repots are to be filed with DesQ and not corestats
	*

	*	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, *
	MA 02110-1301, USA.
	*
*/

#include <DesQUtils.hpp>
#include "DesQUsagePie.hpp"

static QList<QColor> colors = {
	0x2ECC71, 0xD35400, 0x3498DB, 0xF1C40F, 0xE67E22,
	0x1ABC9C, 0x9B59B6, 0x34495E, 0xE74C3C, 0xC0392B,
	0x8E44AD, 0xFF8F00, 0xEF6C00, 0x4E342E, 0x424242,
	0x5499C7, 0x58D68D, 0xCD6155, 0xF5B041, 0x566573
};

DesQUsagePie::DesQUsagePie( QString resource, int segments, QWidget *parent ) : QWidget( parent ) {

	mSegments = segments;
	mUnits = DesQUsagePie::PERCENT;
	resName = QString( resource );

	setMinimumSize( QSize( 150, 100 ) );

	setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding );

	for ( int i = 0; i < segments; i++ ) {
		mValues.append( PieResourceValue{ 2, 1 } );
	}
};

void DesQUsagePie::setUnits( DesQUsagePie::Units units ) {

	mUnits = units;
	repaint();
};

void DesQUsagePie::setLabels( QStringList labels ) {

	mLabels.clear();
	mLabels << labels;

	QFontMetrics fm( font() );
	labelHeight = fm.lineSpacing() * labels.count() + ( labels.count() - 1 ) * 5;

	repaint();
};

void DesQUsagePie::setValues( PieResourceValues values ) {

	mValues.clear();
	mValues << values;

	mMax = 0;
	Q_FOREACH( PieResourceValue res, mValues )
		mMax += res.total;

	repaint();
};

QSize DesQUsagePie::sizeHint() {

	int x = qMin( width(), height() );
	return QSize( x, x );
};

QSize DesQUsagePie::minimumSizeHint() {

	return QSize( minWidth, minWidth );
};

int DesQUsagePie::minimumWidth() {

	return minWidth;
};

void DesQUsagePie::paintEvent( QPaintEvent *pEvent ) {

	int side = qMin( width() - 50 - 10, height() - 10 );
	QImage img( side, side, QImage::Format_ARGB32 );
	img.fill( Qt::transparent );

	QPainter painter( &img );
	painter.setRenderHints( QPainter::Antialiasing | QPainter::TextAntialiasing );

	painter.save();
	QPalette pltt = qApp->palette();
	QColor bg = pltt.color( QPalette::Highlight );
	bg.setAlpha( 27 );
	painter.setPen( Qt::NoPen );
	painter.setBrush( bg );
	painter.drawEllipse( QPointF( side / 2, side / 2 ), side / 2 - 1, side / 2 - 1 );
	painter.restore();

	/* Start at 12 o'clock */
	double startAngle = 90.0 * 16.0;

	for ( int s = 0; s < mSegments; s++ ) {
		painter.save();

		painter.setPen( Qt::NoPen );
		painter.setBrush( colors.at( s ) );

		double spanAngle = -360.0 * 16 * mValues.value( s ).total / mMax;

		double x = 3.0;
		double y = 3.0;

		painter.drawPie( QRectF( x, y, side - 2 * x, side - 2 * y ), startAngle, spanAngle );

		painter.setBrush( colors.at( s ).darker( 110 ) );

		srand( time( 0 ) );
		double usage = 1.0 * mValues.value( s ).used / mValues.value( s ).total;
		painter.drawPie( QRectF( x, y, side - 2 * x, side - 2 * y ), startAngle, spanAngle * usage );

		painter.restore();

		startAngle += spanAngle;
	}

	QString text;
	switch( mUnits ) {
		case DesQUsagePie::PERCENT: {
			text = "%";
			break;
		}

		case DesQUsagePie::SIZE: {
			text = DesQUtils::formatSizeStr( mMax );
			break;
		}

		case DesQUsagePie::SPEED: {
			text = DesQUtils::formatSizeStr( mMax ) + "/s";
			break;
		}
	}

	painter.setPen( Qt::NoPen );
	painter.setBrush( pltt.color( QPalette::HighlightedText ).darker() );
	painter.drawEllipse( QPoint( side / 2, side / 2 ), 25, 25 );

	painter.save();
	painter.setPen( pltt.color( QPalette::HighlightedText ) );
	painter.setFont( QFont( font().family(), 8 ) );
	painter.drawText( QRectF( 0, 0, side, side ), Qt::AlignCenter, resName + "\n" + text );
	painter.restore();

	painter.end();

	painter.begin( this );
	painter.setRenderHints( QPainter::Antialiasing );
	painter.fillRect( rect(), QColor( 0, 0, 0, 10 ) );
	painter.fillRect( rect(), QColor( 255, 255, 255, 10 ) );
	painter.drawImage( QRect( 5, ( height() - side ) / 2, side, side ), img );

	painter.save();
	int y = ( height() - labelHeight ) / 2 - 5;			// We will be adding 5, so reduce that amount at first.
	y = ( y > 0 ? y : 0 );
	QRectF bound( 0, y, 0, 0 );
	for( int s = 0; s < mLabels.count(); s++ ) {
		painter.setPen( colors.value( s ) );
		painter.drawText( QRect( side + 5, bound.bottom() + 5, 45, height() ), Qt::AlignTop | Qt::AlignRight, mLabels.value( s ), &bound );
	}
	painter.restore();

	painter.end();

	pEvent->accept();
};
